#!/usr/bin/perl

package Advisor;

##############################################################
## This package consists of logic for giving recomendations	##
## The function accepts the type of module and the sub type	##
## within the the module as parameter and then decides on	##
## the value to recommend and then returns the value. This	##
## function is called from Document::updateModuleNode()		##
## while updating the module nodes.							##
##############################################################


use strict;
use global;

sub getRecommendation {
	# get whether it is for system, jvm, WAS or Hardware
	my $type = shift;
	# get the sub type within the module like version in jvm..
	my $subType = shift;
	# decide the next parameters on these 2 and give recomendation
	if ($type eq "Hardware") {
		if ($subType eq "Partition Virtual Processors") {
			my $procSharing = $global::Modules{"Hardware"}->{"Partition Processor Sharing"};
			my $activeProc = $global::Modules{"Hardware"}->{"Partition Active Processors"};
			my $virtualProc = $global::Modules{"Hardware"}->{"Partition Virtual Processors"};
			my $totalProc = $global::Modules{"Hardware"}->{"Total System Processors"};
			my $processortype= $global::Modules{"Hardware"}->{"Processor Family"};
			my $value;
			if ($processortype eq "POWER 7") {
				$value=8;
			}else {
				$value=2;
			}		
			
			if ($procSharing eq "Shared Capped") {
					return (int($activeProc),int($activeProc));
			} elsif($procSharing eq "Shared Uncapped") {
				if ($virtualProc < $activeProc) {
					return (int($activeProc),((int($activeProc)>$value)?int($activeProc):$value));
				} elsif ($virtualProc > $totalProc) {
					return (int($totalProc),(($value>int($totalProc))?$value:int($totalProc)));
				} else {
					return (int($virtualProc),int($virtualProc));
				}
			} else {
				#dedicated
				return (int($activeProc),int($activeProc));
			}
		} elsif ($subType eq "Partition Processor Sharing") {
			my $procSharing = $global::Modules{"Hardware"}->{"Partition Processor Sharing"};
			if ($procSharing eq "Shared Capped") {
				return "Dedicated";
			}
			return $procSharing;
		} elsif($subType eq "Partition Memory"){
			if($global::Modules{"Hardware"}->{"Free Memory On Partition"}==0){
				return "Add additional memory for the partition";
			}
			return $global::Modules{"Hardware"}->{"Partition Memory"}; 
		}
	} elsif ($type eq "AIX") {
		if ($subType eq "SMT Level") {
			my $procType = $global::Modules{"Hardware"}->{"Processor Family"};
			if ($procType eq "POWER 7") {
				return 4;
			}
			return 2;
		} elsif($subType eq "RSET Information for job") {
			if ($global::Modules{"Hardware"}->{"Partition Number of Chips"} > 1) {
				return "More Details...";
			}
			return $global::Modules{"AIX"}->{"RSET Information for job"};
		} elsif ($subType eq "Reserved 16MB Large Pages") {
			return "";	# do not recommend anything for now
		} elsif($subType eq "TCP Buffer Size (Send)" or $subType eq "TCP Buffer Size (Receive)"){
			my $wasJob=$global::Modules{"WebSphere"}->{"IsValid"};
			if($wasJob == 1){
				return(64,140);
			}
			else{
				return(14,140);
			}
		} 
		elsif($subType eq "AIXTHREAD_SCOPE environment variable") {
						return "S";
		} elsif($subType eq "MEMORY_AFFINITY environment variable"){
				my $Env_MEMMORY=$global::Modules{"AIX"}->{"MEMORY_AFFINITY environment variable"};
				#if($Env_MEMMORY ne "Not Set"){
					my $rset=$global::Modules{"AIX"}->{"RSET Information for job"};
					if($rset ne "Off"){
						return "MCM";
					}
				#}
				return "Not Set";
						
		 } elsif($subType eq "MALLOCOPTIONS environment variable"){
		 		return "";
		 }  elsif($subType eq "SPINLOOPTIME environment variable"){
		 		#my $Env_SPIN=$global::Modules{"AIX"}->{"SPINLOOPTIME enviornment variable"};
		 		return "";
		 } elsif($subType eq "LDR_CNTRL environment variable"){
		 		#my $Env_LD=$global::Modules{"AIX"}->{"LDR_CNTRL environment variable"};
		 		return "";
		 } elsif($subType eq "EXTSHM environment variable"){
		 	my $Env_EXTSHM=$global::Modules{"AIX"}->{"EXTSHM environment variable"};
		 	if($Env_EXTSHM ne "Not Set"){
		 		my $resvered=$global::Modules{"AIX"}->{"Reserved 16MB Large Pages"};
		 		my $wasJob=$global::Modules{"WebSphere"}->{"IsValid"};
		 		my $large=$global::Modules{"Java"}->{"Use 16 MB Large Pages"};
		 		if($resvered >0 && $wasJob==1 && $large eq "Yes"){
		 			return "off";
		 		}
			}
		 		elsif ($Env_EXTSHM eq "Not Set"){
		 			return "Not Set";
		 	}
		 	
		 			return "ON";
				
		}
		}elsif ($type eq "Java") {
		if ($subType eq "JVM Type") {
			my $jvmType = $global::Modules{"Java"}->{"JVM Type"};
			my $maxHeap = $global::Modules{"Java"}->{"Maximum Heap Size"};
			my $wasJob=$global::Modules{"WebSphere"}->{"IsValid"};
			if ($jvmType == 64 && $maxHeap < 3 &&($wasJob==1 && $maxHeap*1024>2048)) {
				return "32";
			}
			else{
				return "32"
			}
			return $jvmType;
		} elsif ($subType eq "Enable Compressed Refs") {
			my $jvmType = $global::Modules{"Java"}->{"JVM Type"};
			my $maxHeap = $global::Modules{"Java"}->{"Maximum Heap Size"};
			if($jvmType == 64 && $maxHeap < 28) {
				return "On";
			}
			return "Off";
		} elsif ($subType eq "Initial Heap Size") {
			my $maxHeap = $global::Modules{"Java"}->{"Maximum Heap Size"};
			return (0.25*$maxHeap, $maxHeap);
		} elsif ($subType eq "Garbage Collection Policy") {
			if (($global::Modules{"Java"}->{"Partition Active Processors"}) >= 16) {
				if (($global::Modules{"Java"}->{"Garbage Collection Policy"}) eq  "optavgpause") {
					return "optthruput";
				} else {
					return $global::Modules{"Java"}->{"Garbage Collection Policy"};
				}
			} elsif (($global::Modules{"Java"}->{"Garbage Collection Policy"}) eq  "optavgpause") {
				return "optthruput";
			} elsif (($global::Modules{"Java"}->{"Garbage Collection Policy"}) eq  "subpool") {
				return "optthruput";
			}
			return $global::Modules{"Java"}->{"Garbage Collection Policy"};
		} elsif ($subType eq "Maximum Heap Size") {
			#all values in GB
			my $current = $global::Modules{"Java"}->{"Maximum Heap Size"};
			my $recommend = 0;
			if ($global::Modules{"Java"}->{"JVM Type"} == 32) {
				if ($global::Modules{"WebSphere"}->{"IsValid"}==1) {
					$recommend = 2;
				} else {
					$recommend = 2.5;
				}
			} else {
				my $recommend = 10;
			}
			if ($current < $recommend) {
				my $temp = ($recommend < $current*2) ? $recommend : $current*2;
				if (($temp - $current) < $global::Modules{"Hardware"}->{"Free Memory On Partition"}) {
					return $temp;
				}
			}
			return $current;
		} elsif ($subType eq "Use 16 MB Large Pages") {
			return "";	#do not recommend anything for now.
		}
	} elsif ($type eq "WebSphere") {
		if ($subType eq "Minimum Web Container Threads") {
			my $logicalProcs = $global::Modules{"Hardware"}->{"Partition Logical Processors"};
			my $NoChips=  $global::Modules{"Hardware"}->{"Partition Number of Chips"};
			my $maxProcs = (10 > (3*($logicalProcs / $NoChips ))) ? 10 : (3*($logicalProcs/ $NoChips ));
			return (10, $maxProcs);
		} elsif ($subType eq "Maximum Web Container Threads") {
                        my $logicalProcs = $global::Modules{"Hardware"}->{"Partition Logical Processors"};
                        my $NoChips=  $global::Modules{"Hardware"}->{"Partition Number of Chips"};
			my $min = (10 > ($logicalProcs / $NoChips )) ? 10 : ($logicalProcs / $NoChips );
			my $max = 6*$min;
			return ($min, $max);
		} elsif ($subType eq "WebSphere Tracing") {
			my $trace = $global::Modules{"WebSphere"}->{"WebSphere Tracing"};
			$trace =~ s/=config/=info/g;
			$trace =~ s/=fine/=info/g;
			$trace =~ s/=finer/=info/g;
			$trace =~ s/=finest/=info/g;
			$trace =~ s/=detail/=info/g;
			return $trace;
		}
	}
	return ("TODO", "TODO");
}

1;
