#!/usr/bin/perl

package DataCollector;


##################################################################
## This package contains functions to collect the required		## 
## metric values from the various categories like jvm, WAS.		##
##################################################################

use strict;

##...........function to collect the system stats and lparstats..........##
sub collectSystemStats {
	if ($global::DEBUG) {
		print "DataCollector.collectSystemStats() : Collecting the Hardware metrices\n";
	}
	## initiate a new associative array to store all the values ##
	## note : use same headers as used in the xml file ##
	my %SystemStats = ("IsValid" => 1);

	## getting cpu information ##
	my $model = `uname -M`;
	chop $model;
	$SystemStats{"Model"} = $model;
	my $serial = `uname -u`;
	chop $serial;
	$SystemStats{"Serial Number"} = $serial;
	my $family = `getsystype -i`;
	chop $family;
	$SystemStats{"Processor Family"} = $family;
	#if ($family eq 'POWER 7') {
		my $count=0;
		$count=`lssrad -a | wc -l`;
		$count=$count - 1;
		$SystemStats{"Partition Number of Chips"}=$count;
	#}
	my @fields = split( / +/, `lsattr -El proc0 | grep frequency` );    # in Hz
	$SystemStats{"Processor Speed"} = sprintf("%.2f", $fields[1] / 1000000000);  # convert to GHz
	#get the number of processor threads
	my @procs = split(/ +/, `lscfg | grep proc | wc -l`);
	chop $procs[1];	#remove newline
	$SystemStats{"Total System Processors"} = $procs[1];
	## getting information related to partition ##
	my @lparstat = `lparstat -i 2>>/dev/null`;
	my @fields = split( / *: */, $lparstat[1] );    # break the line at ':' (with trailing spaces)
	chop $fields[1];
	$SystemStats{"Partition Name"} = $fields[1];
	my @fields = split( / *: */, $lparstat[4] );
	chop $fields[1];
	my $shareMode = $fields[1];	
	my @fields = split( / *: */, $lparstat[3] );
	chop $fields[1];
	if ($fields[1] =~ "Dedicated") {
		$SystemStats{"Partition Processor Sharing"} = "Dedicated";
	} else {
		$SystemStats{"Partition Processor Sharing"} = "Shared ".$shareMode;
	}
	## checking the processor folding policy
	# get the "Power saving mode"
	
	my @foldPolicy = split(" = ", `schedo -o vpm_fold_policy`);
	chop $foldPolicy[1];
	my $folding = "Disabled";
	if (($SystemStats{"Partition Processor Sharing"} =~ /Shared/) && ($foldPolicy[1] & 1)) {
		# if the proc sharing is Shared and first bit is set
		$folding = "Enabled";
	} elsif (($SystemStats{"Partition Processor Sharing"} eq "Dedicated") && ($foldPolicy[1] & 2)) {
		# if the proc sharing is dedicated and second bit is set
		$folding = "Enabled";
	}
	my @fields = split( / *: */, $lparstat[18] );
	chop $fields[1];
	$SystemStats{"Partition Maximum Processors"} = $fields[1];
	my @fields = split( / *: */, $lparstat[19] );
	chop $fields[1];
	$SystemStats{"System Active Processors"} = $fields[1];
	
	my @fields = split( / *: */, $lparstat[5] );
	chop $fields[1];
	if($fields[1] == (int $fields[1])) {
        $SystemStats{"Partition Active Processors"} = int $fields[1];
    } else {
        $SystemStats{"Partition Active Processors"} = $fields[1];
    }
	my @fields = split( / *: */, $lparstat[9] );
	chop $fields[1];
	my $virProc = $fields[1];
	if ($SystemStats{"Partition Processor Sharing"} =~ /Shared/) {
		my @fields = split( / *: */, $lparstat[6] );
		chop $fields[1];
		$SystemStats{"Number of Processors in Shared Pool"} = $fields[1];
		$SystemStats{"Partition Virtual Processors"} = $virProc;
	}
	
	my $SmtLevel = `lsattr -El proc0 | grep smt_threads`;
	chop $SmtLevel;
	my @Levels = split( / +/, $SmtLevel );
    $SystemStats{"Partition Logical Processors"}= $virProc * $Levels[1];
	my @fields    = split( / *: */, $lparstat[11] );    ## gives 123 MB
	my @memNumber = split( ' ',     $fields[1] );       ## get only the number
	$SystemStats{"Partition Memory"} = sprintf("%.2f", $memNumber[0]/1024 );    # convert to GB
	##calculating Free memory
	my @memory = split(/ +/, `svmon -G -O unit=GB | grep ^memory`);
	my $freeMemory = 0;		#memory[2] = used Total Memory (Ture memory + AME)
	if ($memory[2] > $SystemStats{"Partition Memory"}) {
		$freeMemory = 0;
	} else {
		$freeMemory = $SystemStats{"Partition Memory"} - $memory[2];
	}
	$SystemStats{"Free Memory On Partition"} = $freeMemory;
	
	if ($global::DEBUG)  {
		print "DataCollector.collectSystemStats() : Collection of Hardware metrices completed\n";
	}
	return %SystemStats;
}
##.................end of function collectSystemStats()..................##


##............function to collect AIX information and OS settings........................##
sub collectOSStats {
	if ($global::DEBUG)  {
		print "DataCollector.collectOSStats() : Collecting of AIX metrices\n";
	}
	## initiate a new associative array to store all the values ##
	## note : use same headers as used in the xml file ##
	my %AixStats = ("IsValid" => 1);
	my $pid = $global::Process{"Id"};
	## getting aix version info ##
	my $version = `uname -v`;
	chop $version;
	my $release = `uname -r`;
	chop $release;
	my $level = `oslevel -r`;
	chop $level;
	$AixStats{"AIX Version"} = $version . "." . $release . " : " . $level;
	my $numFD = `ulimit -n`;
	chop $numFD;
	$AixStats{"Maximum Number File Descriptors"} = $numFD;
	my $data = `ulimit -d`;
	chop $data;
	$AixStats{"Maximun Data"} = $data;
	my @ulimit=`ulimit -Ha`;
	my @field=split( / *  */,$ulimit[2]);
	chop($field[1]);
	$AixStats{"User Data Area Size"}=$field[1];
	my @field=split( / *  */,$ulimit[3]);
	chop($field[1]);
	$AixStats{"User Stack Size"}=$field[1];
	my @field=split( / *  */,$ulimit[4]);
	chop($field[1]);
	$AixStats{"User Memory Size"}=$field[1];
	my @TcpIdle = split( /= +/, `no -a | grep tcp_keepidle` );
	chop $TcpIdle[1];
	$AixStats{"TCP_KeepIDLE"} = $TcpIdle[1];
	my @TcpBuff = split( /= +/, `no -a | grep tcp_sendspace` );
	chop $TcpBuff[1];
	$TcpBuff[1] = $TcpBuff[1] / 1024;
	if($TcpBuff[1] != int $TcpBuff[1]){
		$AixStats{"TCP Buffer Size (Send)"} = sprintf("%.2f", $TcpBuff[1]);	#to KB
	}else{
		$AixStats{"TCP Buffer Size (Send)"} = sprintf("%d", $TcpBuff[1]);
	}
	my @TcpBuff = split( /= +/, `no -a | grep tcp_recvspace` );
	chop $TcpBuff[1];
	$TcpBuff[1] = $TcpBuff[1] / 1024;
	if($TcpBuff[1] != int $TcpBuff[1]){
		$AixStats{"TCP Buffer Size (Receive)"} = sprintf("%.2f", $TcpBuff[1]);	#to KB
	}else{
		$AixStats{"TCP Buffer Size (Receive)"} = sprintf("%d", $TcpBuff[1]);
	}
	
	my $SmtLevel = `lsattr -El proc0 | grep smt_threads`;
	chop $SmtLevel;
	my @Levels = split( / +/, $SmtLevel );
	$AixStats{"SMT Level"} = $Levels[1];
	my @Rset = `lsrset -v -p $pid`;
	if ( $Rset[0] =~ /^Default/ ) {
		$AixStats{"RSET Information for job"} = "Off";
	}
	else {
		$AixStats{"RSET Information for job"} = "On";
	}
	## get the number of 16M pages
	my @pages = `vmstat -P 16m`;
	chop $pages[-1];	#remove newline
	my @pageList = split(/ +/, $pages[-1]);
	$AixStats{"Reserved 16MB Large Pages"} = $pageList[2];
	
	if ($global::DEBUG)  {
		print "DataCollector.collectOSStats() : Collection of AIX metrices completed\n";
	}
	
	return %AixStats;
}
##.............end of function collectOSStats()...........................................##


##.............function to collect WAS information and settings..............##
sub collectWASStats {
		if ($global::DEBUG)  {
			print "DataCollector.collectWASStats() : Collecting WAS metrices\n";
		}
        my $is_WAS_JOB=0;
        my @profilepath;
        my $path;
        my $new_path;
        my $servername;
        my $wsadminPath;
        my $versionPath;
        ## try to get the process command line options
        my $CommandOutput =$global::Process{"CommandLine"};
        ## initiate a new associative array to store Websphere values ##
        my %WebsphereStats = ("IsValid" => 1);
        ## split the webpshere command to get the command line options given
		my @CommandLineArgs = split(" ", $CommandOutput);

        for (my $i=0; $CommandLineArgs[$i] ne ""; $i++) {
                #To get the profile path and to set the Environment variable PATH
                if( $CommandLineArgs[$i]=~ /^-Duser.install.root/) {
                        @profilepath= split("=",$CommandLineArgs[$i]);
                        $path=$ENV{'PATH'};
                        $new_path=$profilepath[1].":".$path;
                        $ENV{'PATH'}=$new_path;
                        $is_WAS_JOB=1;
                }
        }

        if(!$is_WAS_JOB) {
        	# if it is not a WAS job just set the flag and return
                $WebsphereStats{"IsValid"}=0;
                return %WebsphereStats;
        }
        
        #To get servername
        $servername=$CommandLineArgs[-1];

        # Set the wsadmin.sh path
        $wsadminPath=$profilepath[1]."/bin/wsadmin.sh";

        # Set the versionInfo.sh path
        $versionPath=$profilepath[1]."/bin/versionInfo.sh";

        # Call jacl script to get the websphere values
        my @jaclOutput=`$wsadminPath -f WAS.jacl $servername`;

        #Store the value in the arrary
        for(my $i=0;$jaclOutput[$i] ne "";$i++)
        {
                my @jaclArgs=split("-->",$jaclOutput[$i]);
                $jaclArgs[1]=~ s/^\s+//; #remove leading spaces
                $jaclArgs[1]=~ s/\s+$//; #remove trailing spaces
                if($jaclArgs[0] =~ /^maxInMemorySessionCount/){
                        $WebsphereStats{"Maximum HTTP Sessions"}=$jaclArgs[1];
                 } elsif($jaclArgs[0] =~ /^WebSphere PMI/){
                        if($jaclArgs[1] =~ m/^true/i){
                                $WebsphereStats{"WebSphere PMI"}="On";
                        } else {
                                $WebsphereStats{"WebSphere PMI"}="Off";
                        }
                 } elsif($jaclArgs[0] =~ /^invalidationTimeout/){
                        if($jaclArgs[1]=~ /^-1/){
                                $WebsphereStats{"Session Time Out"}="No TimeOut";
                        } else {
                                $WebsphereStats{"Session Time Out"}=$jaclArgs[1];
                        }
                 } elsif($jaclArgs[0] =~ /^ThreadPool MinSize/){
                        $WebsphereStats{"Minimum Web Container Threads"}=$jaclArgs[1];
                 } elsif($jaclArgs[0] =~ /^ThreadPool MaxSize/){
                        $WebsphereStats{"Maximum Web Container Threads"}=$jaclArgs[1];
                 } elsif($jaclArgs[0]=~ /^WebSphere Tracing/){
                 		$jaclArgs[1] =~ s/:/ /g;	#this is used just for formatting
                        $WebsphereStats{"WebSphere Tracing"} = $jaclArgs[1];
                 }
        }
        # To get the Websphere version number
        my @WebsphereVersion=`$versionPath | grep "Version   "`;
        my @WebsphereArg=split(" ",$WebsphereVersion[0]);
        $WebsphereStats{"WebSphere Version"}=$WebsphereArg[1];

        #Restore the PATH value
        $ENV{'PATH'}=$path;

		if ($global::DEBUG)  {
			print "DataCollector.collectWASStats() : Collection of WAS metrices completed\n";
		}
        return %WebsphereStats;

}
##.............end of function collectWASStats().............................##


##.............function to get the ENV details from the dump .................##
sub updateEnvStats {
	my $module = shift;
	my $thisModule = $global::Modules{$module};
	my $pid = $global::Process{"Id"};
	my $dumpFile = global::getDumpFile($pid);
	if ($dumpFile eq "") {
		print "Warning : Could not capture the java coredump file.\n";
		return ;
	} 
	if ($global::DEBUG) {
		print "DataCollector.updateEnvStats() : Obtained the dump file $dumpFile\n";
		print "DataCollector.updateEnvStats() : Updating the $module module with ENVars\n";
	}
	
	#set to NOT SET
	$thisModule->{"AIXTHREAD_SCOPE environment variable"} = "Not Set";
	$thisModule->{"SPINLOOPTIME environment variable"} = "Not Set";
	$thisModule->{"LDR_CNTRL environment variable"} = "Not Set";
	$thisModule->{"EXTSHM environment variable"} = "Not Set";
	$thisModule->{"MEMORY_AFFINITY environment variable"} = "Not Set";
    $thisModule->{"MALLOCOPTIONS environment variable"}="Not Set";
	open(CORE, $dumpFile) or die "IOError: Could not open the dump file : $dumpFile\n";
	while(($_ = <CORE>)){
		chop $_;                #remove trailing newline
		if ($_ =~ /AIXTHREAD_SCOPE=/) {
			my @var = split("AIXTHREAD_SCOPE=", $_);
			$thisModule->{"AIXTHREAD_SCOPE enviornment variable"} = $var[1];
		} elsif ($_ =~ /SPINLOOPTIME=/) {
			my @var = split("SPINLOOPTIME=", $_);
			$thisModule->{"SPINLOOPTIME enviornment variable"} = $var[1];
		} elsif ($_ =~ /LDR_CTL=/) {
			my @var = split("LDR_CNTRL=", $_);
			$thisModule->{"LDR_CNTRL environment variable"} = $var[1];
		} elsif ($_ =~ /EXTSHM=/) {
			my @var = split("EXTSHM=", $_);
			$thisModule->{"EXTSHM environment variable"} = $var[1];
		} elsif ($_ =~ /MEMORY_AFFINITY=/) {
			my @var = split("MEMORY_AFFINITY=", $_);
			$thisModule->{"MEMORY_AFFINITY enviornment variable"} = $var[1];
		} elsif($_ =~ /MALLOCOPTIONS=/){
			my @var=split("MALLOCOPTIONS=",$_);
			$thisModule->{"MALLOCOPTIONS environment variable"}= $var[1];
		}
	}
}

##.............function to get the JVM settings comandline options............##
sub collectJVMStats {
	if ($global::DEBUG)  {
		print "DataCollector.collectJVMStats() : Collecting JVM metrices\n";
	}
	
	my $CommandOutput = $global::Process{"CommandLine"};
	## split the jvm command to get the command line options given
	my @CommandLineArgs = split( " ", $CommandOutput );
	
	## initiate a new associative array by setting system default values ##
	## note : use same headers as used in the xml file ##
	my %JVMStats = _getJVMDefaults($CommandLineArgs[0]);
    
    ## Setting Default value for gcthreads ##
    $JVMStats{"Xgcthreads"}="Default";
       
	for ( my $i = 0 ; $CommandLineArgs[$i] ne "" ; $i++ ) {
		if ( $CommandLineArgs[$i] =~ /^-Xgcpolicy/ ) {
			$JVMStats{"Garbage Collection Policy"} =
			  substr( $CommandLineArgs[$i], 11 );
		}
		elsif ( $CommandLineArgs[$i] =~ /^-Xdbg/ ) {
			$JVMStats{"JVM Debug"} = "On";
		}
		elsif ( $CommandLineArgs[$i] =~ /^-verbose/ ) {

			#check the options after verbose. -verbose:[gc,class|...]
			if ( $CommandLineArgs[$i] =~ /gc/ ) {
				$JVMStats{"Verbose Garbage Collection"} = "On";
			}
			if ( $CommandLineArgs[$i] =~ /class/ ) {
				$JVMStats{"Verbose Class Loading"} = "On";
			}
		}
		elsif (($CommandLineArgs[$i] =~ /-Xrun/) || ($CommandLineArgs[$i] =~ /-agentlib/)) {
			$JVMStats{"Java JVMPI"} = "On";
		}
		elsif (( $CommandLineArgs[$i] eq "-Xcompressedrefs" )
			|| ( $CommandLineArgs[$i] eq "-XX:+UseCompressedOops" ) )
		{
			# set this only if it is a 64 bit
			if ($JVMStats{"JVM Type"} == 64) {
				$JVMStats{"Enable Compressed Refs"} = "On";
			}
		}
		elsif ( $CommandLineArgs[$i] =~ /^-Xms/ ) {
			$JVMStats{"Initial Heap Size"} =
			  substr( $CommandLineArgs[$i], 4, -1 );
			my $unit = substr( $CommandLineArgs[$i], -1 );
			#convert to gb
			if ( $unit eq "m" ) {
				$JVMStats{"Initial Heap Size"} /= 1024;
			}
			elsif ( $unit eq "k" ) {
				$JVMStats{"Initial Heap Size"} /= ( 1024 * 1024 );
			}
		}
		elsif ( $CommandLineArgs[$i] =~ /^-Xmx/ ) {
			$JVMStats{"Maximum Heap Size"} =
			  substr( $CommandLineArgs[$i], 4, -1 );
			my $unit = substr( $CommandLineArgs[$i], -1 );

			#convert to gb
			if ( $unit eq "m" ) {
				$JVMStats{"Maximum Heap Size"} /= 1024;
			}
			elsif ( $unit eq "k" ) {
				$JVMStats{"Maximum Heap Size"} /= ( 1024 * 1024 );
			}
		}
		elsif ( $CommandLineArgs[$i] =~ /^-Xlp/ ) {

			#check for alp/flp to see if largepage is set
			my @vmstat = `vmstat -l`;
			my @lgpg   = split( / +/, $vmstat[-1] );
			my $isSet  = 1;
			if ( $lgpg[-1] == 0 && $lgpg[-2] == 0 ) {
				$isSet = 0;
			}
			## now check for the command line options
			if ($CommandLineArgs[$i] eq "-Xlp" ) {
				if ($isSet) {
					# if -Xlp is specified and lp is set the 16M is taken
					$JVMStats{"Use 64K LargePages"}   = "No";
					$JVMStats{"Use 16 MB Large Pages"} = "Yes";	
				} else {
					# if -Xlp is specified and lp is not set  then next large size 64k is taken
					$JVMStats{"Use 64K LargePages"}   = "Yes";
					$JVMStats{"Use 16 MB Large Pages"} = "No";
				}
			} elsif (($CommandLineArgs[$i] eq "-Xlp16m" ) && $isSet ) {
				## if -Xlp16m is specified and lp is set
				$JVMStats{"Use 64K LargePages"}   = "No";
				$JVMStats{"Use 16 MB Large Pages"} = "Yes";
			} elsif ($CommandLineArgs[$i] eq "-Xlp64k" )	{
				## if directly 64k is specified
				$JVMStats{"Use 64K LargePages"} = "Yes";
				$JVMStats{"Use 16 MB Large Pages"} = "No";
			} elsif ($CommandLineArgs[$i] eq "-Xlp4k" )	{
				## if 4k is specified
				$JVMStats{"Use 64K LargePages"} = "No";
				$JVMStats{"Use 16 MB Large Pages"} = "No";
			}
		}
		elsif ( $CommandLineArgs[$i] eq "-Xquickstart" ) {
			$JVMStats{"QuickStart"} = "On";
		}
		elsif ( $CommandLineArgs[$i] eq "-Xgcthreads"){
			$JVMStats{"Xgcthreads"}=$CommandLineArgs[$i+1];
		}
	}    # end of for

	my $pid = $global::Process{"Id"};
	# Get the priority (nice value)
	my $nice = `ps -p $pid -o nice | tail -1`;
	chop $nice;
	$JVMStats{"Job Priority"} = $nice;
	
	# Get the Total Number of JVMs
	my @no_jvm = split(/ +/, `ps -ef | grep java | grep -v grep | wc -l`);
	chop $no_jvm[1];
	$JVMStats{"Number Of JVMs"} = $no_jvm[1] - 1;	# subtract one to remove the current pid

	#Total Max heap
	my $total_heapsize=0;
	my @CommandOutput =`ps -eaf | grep java | grep -v grep | grep -v $pid |  grep -v jpa.pl`;
	my %JVMHeapSize=_loadDefaultHeapSize();
	for(my $i=0;$CommandOutput[$i] ne "" ;$i++)	{
		my @CommandLineArgs = split( " ", $CommandOutput[$i] );
		my $version=_getJavaVersion($CommandLineArgs[8]);
        my $maxheap=$JVMHeapSize{$version};
        for(my $j=0;$CommandLineArgs[$j] ne "";$j++){
        	if ( $CommandLineArgs[$j] =~ /^-Xmx/ ) {
            	$maxheap=substr( $CommandLineArgs[$j], 4, -1 );
                my $unit = substr( $CommandLineArgs[$j], -1 );
				#convert to gb
        	    if ( $unit eq "m" ) {
                	$maxheap /= 1024;
                } elsif ( $unit eq "k" ) {
                	$maxheap /= ( 1024 * 1024 );
                }
            }
        }
        $total_heapsize += $maxheap;
	}
	$JVMStats{"Total Heap Of All JVMs"} = $total_heapsize;
	
	if ($global::DEBUG)  {
		print "DataCollector.collectJVMStats() : Collection of JVM metrices completed\n";
	}
	return %JVMStats;
}
##.................end of function collectJVMStats().............................##


##................this is a package private function that gives JVM default values.......##
sub _getJVMDefaults {
	if ($global::DEBUG)  {
		print "DataCollector._getJVMDefaults() : Getting the default JVM values\n";
	}
	## initiate a new associative array by setting system default values ##
	## note : use same headers as used in the xml file ##
	## get the java path as parameter
	my $command = shift;
	my @jv      = `$command -version 2>&1`;
	my $version = substr( $jv[0], 14, -2 );
	my $type    = 64;
	if ( $jv[2] =~ /ppc-32/ ) {
		$type = 32;
	}
	my $serviceRelease = $jv[1];
	$serviceRelease =~ s/^(.*SR)([0-9]*)(.*)$/\2/g;
	chop $serviceRelease;
	 
	my %JVMStats = (
		"IsValid"					  => 1,
		"JVM Type"                    => $type,
		"JVM Version"                 => $version." SR".$serviceRelease,
		"JVM Release"                 => $serviceRelease,
		"JVM Level"					  => $version,
	);
	#read from file
	my $jvmFile = "jvm.settings";
	open(JVM, $jvmFile) or 
		die "IOError: Could not open the jvm settings file\n";
	my $read = 0; #read only after a version match
	my $end = 0; # flag, if required info is found
	while (!$end) {
		$_ = <JVM>;
		if (!$_) {
			$end=1;
		} else {
			chop $_;		#remove trailing newline
			s/[\t ]+/ /g;	#replace multiple spaces with one
			s/$[ \t]+//g;	#remove leading spaces
			s/^[\t ]+//g;	#remove trailing spaces
			if ($read) {
				if ($_ eq "") {
					$end = 1;
				} else {
					my @metric = split (/ *= */, $_);
					
					if ($#metric != 1) {
						print STDERR "ParseError: jvm settings file not in proper format.\n";
						exit;
					}
					$JVMStats{$metric[0]} = $metric[1];
				}
			} elsif ($_ eq $version) {
				$read = 1;
			}
		}
	}
	close JVM;
	#check for 64K page size based on the SR
	if (($version eq "1.6.0") && ($serviceRelease>7)) {
		$JVMStats{"Use 64K Large Pages"} = "Yes";
	}
	if (!$read) {
		# if read is not set then version not matched
		print STDERR "Error: Could find the defaults for the version $version\nPlease update the jvm.settings file\n";
		exit;
	}
	return %JVMStats;
}
##.....................end of function _getJVMDefaults()...........................##


##.................... function to get Java version ..............................##
sub _getJavaVersion{
        my $command = shift;
        my @jv      = `$command -version 2>&1`;
        my $version = substr( $jv[0], 14, -2 );
        return $version;
}
##.................... end of function  ..............................##

##.................... function to load Default Heap Size for various Java version ..............................##
sub _loadDefaultHeapSize{

        #read from file
        my %HeapSize;
        my $version=0;
        my $heapsize=0;
        my $i=0;
        my $jvmFile = "jvm.settings";
        open(JVM, $jvmFile) or die "IOError: Could not open the jvm settings file\n";

        while(($_ =<JVM>)){
        	chop $_;                #remove trailing newline
        	s/[\t ]+/ /g;   #replace multiple spaces with one
            s/$[ \t]+//g;   #remove leading spaces
            s/^[\t ]+//g;   #remove trailing spaces
            if($_ =~ m/=/) {
            	my @metric = split (/ *= */, $_);
                if ($#metric != 1) {
                	print STDERR "ParseError: jvm settings file not in proper format.\n";
                    exit;
                }
                if($_ =~ /^Maximum Heap Size/) {
                	$heapsize=$metric[1];
                }
            } else {
            	$version=$_;
            }
            if($heapsize!=0 && $version!=0) {
            	$HeapSize{$version}=$heapsize;
                $version=0;
                $heapsize=0;
            }
        }
        close JVM;
        return %HeapSize;
}
##.................... function to load Default Heap Size for Java version ..............................##

1;
