#!/usr/bin/perl

package global;

######################################################
## This package declares and defines the global		##
## variables that are used accross the tool			##
## globally. There are some constants and some are	##
## set in differnt packages and used in some other	##
######################################################

use strict;

## version number ##
our $VERSION = "1.0";
## version ##

## some globals defined here ##
our $INIT_SCORE = 6;					# initial score for a metric
our $MIN_VAL_REC = 5; 					# minimum score to attain to given recommendation
our $INPUT_FILE = "jpa_input.xml";
our $DEBUG = 0;							# set this to print debug messages
our $CERT_FILE = "licence.cert";
our $INFINITE = 99999999;				# some big number to signify infinity
our $SHOW_HIDDEN = 0;					# if this flag is set then hidden fields are shown in the o/p
## end of globals

## get the terminal size here ##
my ($rows, $cols) = split(/ +/, `stty size`);
our $TERM_COLS = $cols;

## Some process relates stats ##
our $Process = ();
our $Hostname;

## Initialize the arguments to default values ##
our $Score = $INIT_SCORE;
our $usageString="
    Java Performance Advisor Tool version $global::VERSION
    Usage: jpa.pl [[-e Beginner|Intermediate|Expert] [-u Test|Production] [-i Primary|Secondary] [-o OutputFile] [-d] pid]
";

our $noPidMessage = "
    Since a pid was not specified, the following list is the current JVMs active on this partition. 
    If the JVM is a WebSphere job, the Profile and Application Server names are shown.
    If this is not a WebSphere JVM, N/A is displayed.

";

## This function is used to get a list of all the jvms
 # This function takes no parameters. This does a ps call
 # parse the class name from all the jvm process and 
 # returns a list of <pid, class, profile(if was)>
 ##
sub getAllJvm {
	my @jvmProcess = `ps -e -o pid,args | grep java | grep -v grep`;
	if ($#jvmProcess < 0) {
		print "There are no java/WAS process currently running in the system\n";
		exit;
	}
	#initialize a list of all the process in pid=>class format
	my %jvmList = ();
	for (my $j=0; $j<=$#jvmProcess; $j++) {
		my @CommandLineArgs = split(" ", $jvmProcess[$j]);
		my $pid = $CommandLineArgs[0];
		my $command = $CommandLineArgs[1];
		my $wasProfile = "N/A";
		my $appServer = "N/A";
		my $className = "";
		for (my $i=2; $i <= $#CommandLineArgs; $i++) {
			if (($CommandLineArgs[$i] eq "-cp") || ($CommandLineArgs[$i] eq "-classpath")) {
				# if the parameter to jvm is -cp then ignore the next param
				$i++;
			} elsif ($CommandLineArgs[$i] =~ /^-Duser.install.root/) {
				# To get the profile name
				$wasProfile = (split("/",$CommandLineArgs[$i]))[-1];
				# This means this is a was process and hence get the server app name
				$appServer = $CommandLineArgs[-1];
			} elsif ($CommandLineArgs[$i] !~ /^-/) {
				# if the argument is non "-" then this is the class name
				$className = $CommandLineArgs[$i];
				#break;
				$i = $#CommandLineArgs;
			}
		}
		# finally add this pid to the list
		$jvmList{$pid} = [$className, $wasProfile, $appServer];
	}
	return %jvmList;
}

## This is the global hash that contains a sub hash
## one for each category##
our $Modules = ();

if ($DEBUG) {
	print "global.pm : Initialized the global variables\n";
}


## this function is maily used for debug
## this function prints an associative array in the proper format
## usage : global::print_array \%array_name
sub print_array {
	my $refArray = shift;
	foreach (keys %{$refArray}) {
		print $_, " : ", $refArray->{$_}, "\n";
	}
	exit;
}

##
 # Function to get the path of the java core file.
 # This needs to be called only after killing the
 # process.  Usage : getDumpFile ($pid)
 ##
sub getDumpFile {
	my $pid= shift;
	my $logfile = "javacore*$pid*txt";

	my ($year,$month,$day,$hour,$min,$sec,$temp) = split ("-", `date +"%Y-%m-%d-%H-%M-%S-"`);
	if ($sec > 57) {
		sleep(4);	#just to prevent cross over of min...
	}
	my ($year,$month,$day,$hour,$min,$sec,$temp) = split ("-", `date +"%Y-%m-%d-%H-%M-%S-"`);
	if (system("kill -QUIT $pid 2>>/dev/null") == -1) {
		print "Warning: Failed sending QUIT signal to process $pid\n";
		return "";
	}
	sleep(2);	#just a buffer to wait for the coredump file to be created
	
	my @CommandOutput = `ps -p $pid -o args 2>>/dev/null`;
	my @CommandLineArgs = split(" ",$CommandOutput[1]);
	for (my $i=1; $CommandLineArgs[$i] ne ""; $i++ ) {
		if ( $CommandLineArgs[$i] =~ /^-Xdump:.*:file=.*/ ) {
			if ($DEBUG) {
				print "global.getDumpFile() : -Xdump specified. Parsing the file name.\n";
			}
			my @dumpoptions = split("file=", $CommandLineArgs[$i]);
			#replace all possible %options
			$logfile = $dumpoptions[1];			$logfile =~ s/%pid/$pid/;			$logfile =~ s/%H/$hour/;
			$logfile =~ s/%M/$min/;			$logfile =~ s/(%S|%seq)/*/;			$logfile =~ s/%Y/$year/;
			$logfile =~ s/%m/$month/;			$logfile =~ s/%d/$day/;
			$i = $#CommandLineArgs;		#just to break
		}
	}
	my @logdir = split("/", $logfile);	#"find" doest not take "folders/file" format in -name
	
	my $coreDir = "";
	my @cwd = split(" ", `procwdx "$pid"`);
	if ($ENV{'IBM_JAVACOREDIR'} ne "") {
		$coreDir = $ENV{'IBM_JAVACOREDIR'};
	} else {
		$coreDir = $cwd[1];
	}
	if ($DEBUG) {
		print "global.getDumpFile() : Corefile Dir = $coreDir, Corefile name = $logfile .\n";
	}
	if ($logfile eq "") {
		return "";
	}
	
	my @fileList = `ls -t $coreDir/$logfile 2>>/dev/null | head -1`;
	if ($#fileList >=0 ) {
		my @fileList = `find $coreDir/ -name "$logdir[-1]" -mmin 1 -print 2>>/dev/null`;	#make sure this is the new one
		if ($#fileList >= 0) {
			chop $fileList[0];
			return $fileList[0];
		}
		if ($DEBUG) {
			print "global.getDumpFile() : File : $logdir[-1] is an old file.\n";
		}
	}
	if ($DEBUG) {
		print "global.getDumpFile() : Executing <find> command.\n";
	}
	@fileList = `find / -name "$logdir[-1]" -mmin 1 -print 2>>/dev/null`;
	my $str = "grep -l CMDLINE ";
	foreach (@fileList) {
		chop $_;
		$str = $str."$_ ";
	}
	$str = $str." 2>>/dev/null";
	@fileList = `$str`;
	if ($#fileList >= 0) {
		$str = "ls -t ";
		foreach (@fileList) {
			chop $_;
			$str = $str."$_ ";
		}
		$str = $str." 2>>/dev/null | head -1";
		@fileList = `$str`;
		if ($#fileList >= 0) {
			chop $fileList[0];
			return $fileList[0];
		}
	}
	return "";
}

1;
