#!/usr/bin/perl

##########################################################
## This file is called from the command line by giving 	##
## command line arguments. This does the validation of	##
## of the given command line arguments, creates a Doc	##
## element, updates it and write the output.			##
##########################################################


use Document;
use strict;
use DataCollector;
use global;


## Initialize the arguments to default values ##
my $adminExperiance = "Beginner";
my $workloadType = "Secondary";
my $machineType = "Production";
my $outputFile = "jpa_output.xml";
my $pid = undef;


## Check if the user has superuser authority ##
if ($global::DEBUG) {
	print "jpa.pl : Checking for root privilages\n";
}
if (`lsuser -a admin \`whoami\` 2>&1 | grep true` eq "") {
	print STDERR "Error : You need admin access in order to use this command. \n";
	exit;
}

## parse the given commandline arguments and validate ##
if ($global::DEBUG) {
	print "jpa.pl : Parsing the commandline arguments\n";
}
if ($#ARGV < 0) {
	print $global::usageString."\n";
	# if no pid is given, display a list of pids to select
	my %jvmList = global::getAllJvm();
	print $global::noPidMessage;
	printf ("%-10s %-45s %-25s %-25s\n\n", "PID", "Class Name", "WAS Profile", "App Server Name");
	foreach (keys %jvmList) {
		printf ("%-10s %-45s %-25s %-25s\n", $_, $jvmList{$_}[0], $jvmList{$_}[1], $jvmList{$_}[2]);
	}
	print "\n\n";
	exit;
} elsif ($#ARGV+1>10) {
	#there should be atmost 10
	print $global::usageString;
	exit;
} elsif ($#ARGV+1 == 1) {
	if(($ARGV[0] eq "-?") || ($ARGV[0] eq "-h")) {
		system("more -sv help.msg");
		exit;
	} elsif ($ARGV[0] eq "-v") {
		print "\nJava Performance Advisor Tool version $global::VERSION\n\n";
		exit;
	}else {
		$pid = $ARGV[0];
	}
} else {
	$pid = $ARGV[$#ARGV];
	for (my $i=0; $i<$#ARGV; $i++) {
		if ($ARGV[$i] eq "-i") {
			# only allowed values are  Primary|Secondary
			$workloadType = $ARGV[++$i];
			if (($workloadType ne "Primary") && ($workloadType ne "Secondary")){
				print $global::usageString;
				exit;
			}
		} elsif ($ARGV[$i] eq "-u") {
			$machineType = $ARGV[++$i];
			# only allowed type are Test|Production
			if (($machineType ne "Test") && ($machineType ne "Production")) {
				print $global::usageString;
				exit;
			}
		} elsif ($ARGV[$i] eq "-e") {
			$adminExperiance = $ARGV[++$i];
			# only allowed type are Beginner|Intermediate|Expert
			if (($adminExperiance ne "Beginner") && ($adminExperiance ne "Intermediate") && ($adminExperiance ne "Expert")) {
				print $global::usageString;
				exit;
			}
		} elsif ($ARGV[$i] eq "-o") {
			$outputFile = $ARGV[++$i];
		} elsif ($ARGV[$i] eq "-internal") {
			$global::SHOW_HIDDEN = 1;
		} elsif($ARGV[$i] eq "-d"){
			$global::DEBUG = 1;
		} 
		else {
			print $global::usageString;
			exit;
		}
	}
}
# validate the pid
if ($pid =~ /^-/) {
	print $global::usageString;
	exit;
}
if ($global::DEBUG) {
	print "jpa.pl : Validating the given pid\n";
}
my @CommandOutput = `ps -p "$pid" -o args 2>>/dev/null`;
if ($CommandOutput[1] eq "") {
	print STDERR "Error: The given process id does not seem to be valid.\n";
	exit;
} elsif ($CommandOutput[1] !~ /java/) {
	print STDERR "Error: The given process id does not seem to be a jvm process.\n";
	exit;
}
## command line argument parsing and validation completed  ##

## Update the process info in the global array ##
$global::Process{"Id"} = $pid;
my @user = split(/ +/, `ps -p $pid -o user | tail -1`);
chop $user[-1];
$global::Process{"User"} = $user[-1];
$global::Process{"CommandLine"} = $CommandOutput[1];
if ($global::DEBUG) {
	print "jpa.pl : Fetched the job info\n";
}
## display the certificate before moving forward ##
if (!$global::DEBUG) {
	open(FH, $global::CERT_FILE) || die("Error : Could not find the licence file\n");
	system("tput smcup; tput cup 0 0;");
	print <FH>;
	close(FH);
	print "\n\nI Agree (y/n) : ";
	chomp(my $option = <STDIN>);
	system("tput rmcup");
	if ($option ne "y" && $option ne "Y") {
		# just quit if the user doesnot agree to the licence
		exit;
	}
}


## The following is the algorithm that decides on the SettingScore ##
if ($global::DEBUG) {
	print "jpa.pl : Setting the globl score\n";
}
if ($adminExperiance eq "Intermediate") {
	$global::Score -= 1;
} elsif ($adminExperiance eq "Beginner") {
	$global::Score -= 2;
}
if ($workloadType eq "Secondary") {
	$global::Score -= 1;
}
if ($machineType eq "Production") {
	$global::Score -= 1;
}
## finding the score is done here ##


## parse the input xml file and load it to memory ##
if ($global::DEBUG) {
	print "jpa.pl : Loading the input xml file\n";
}
my $doc = new Document($global::INPUT_FILE);

## update the UserProfile node here ##
if ($global::DEBUG) {
	print "jpa.pl : Updating the user profile in the xml file\n";
}
$doc->updateProfileNode($adminExperiance, $machineType, $workloadType, );

## get the Hardware details ##
if ($global::DEBUG) {
	print "jpa.pl : Processing the Hardware module\n";
}
my %SystemStats = DataCollector::collectSystemStats();
$global::Modules{"Hardware"} = \%SystemStats;
## get the AIX details ##
if ($global::DEBUG) {
	print "jpa.pl : Processing the AIX module\n";
}
my %OSStats = DataCollector::collectOSStats();
$global::Modules{"AIX"} = \%OSStats;
## get the ENV details ##
if ($global::DEBUG) {
	print "jpa.pl : Updating ENV vars\n";
}
DataCollector::updateEnvStats("AIX");
## get the JVM status ##
if ($global::DEBUG) {
	print "jpa.pl : Processing the JVM module\n"; 
}
my %JVMStats = DataCollector::collectJVMStats();
$global::Modules{"Java"} = \%JVMStats;
## get the WAS details if it is a WAS pid ##
if ($global::DEBUG) {
	print "jpa.pl : Processing the WebSphere module\n";
}
my %WASStats = DataCollector::collectWASStats();
$global::Modules{"WebSphere"} = \%WASStats;

## update the xml with recommendations ##
if ($global::DEBUG) {
	print "jpa.pl : Updating the XML Files with recommendations\n";
}
$doc->updateModuleNode('Hardware');
$doc->updateModuleNode('AIX');
$doc->updateModuleNode('Java');
$doc->updateModuleNode('WebSphere');	

## write the ouput to the given file ##
if ($global::DEBUG) {
	print "jpa.pl : Writing the output to $outputFile\n";
}
$doc->printDoc($outputFile);
